#### H(y)=0 if y<=0, exp(y) otherwise#####

####packages####
library(rms)
library(mlt)
library(tram)
library(multcomp)
library(parallel)
library(tidyverse)
library(magrittr)

####functions####
#source('~/Box Sync/Vanderbilt/BryanProject/sim/sim1009/sim_function.R')
source('~/simulation/sim10000/sim_function.R')

###number of replication#####
reps <- 10000

####stoer the summary data
names <- c('n50','n100','n500','n1000')
sum_log <- vector("list",4); names(sum_log) <- names #log-likelihood

###values for quantile and cdf
val_q <- c(0.1, 0.5, 0.8)
val_c <- 3 #cdf

###store result
res_orm <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),1),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),2),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),6),
             0)
names(res_orm) <- c('beta','c_0','c_1','q1_0','q1_1','q5_0','q5_1','q8_0','q8_1','loglikelihood')

res_mlt <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),1),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),2),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),6),
             0)
names(res_mlt) <- c('beta','c_0','c_1','q1_0','q1_1','q5_0','q5_1','q8_0','q8_1','loglikelihood')

res_mlt_surv <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),1),
                  rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),2),
                  rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),6),
                  0)
names(res_mlt_surv) <- c('beta','c_0','c_1','q1_0','q1_1','q5_0','q5_1','q8_0','q8_1','loglikelihood')


###True values using simulation (m=mean,q=quantile,c=cdf)
beta <- 1
set.seed(35)
x <- rbinom(10000000,1,p=0.5)
e <- rnorm(10000000,0,1)
y <- ifelse((x+e)>0, x+e, 0)
data <- data.frame('x'=x,'y'=y)
true_1_q <- quantile(data$y[data$x==1],val_q)
true_0_q <- quantile(data$y[data$x==0],val_q)
true_1_c <- sum(data$y[data$x==1]<=val_c)/length(data$y[data$x==1])
true_0_c <- sum(data$y[data$x==0]<=val_c)/length(data$y[data$x==0])


####n=50####
n <- 50

####set up cluster###
cl <- makeCluster(detectCores()-1)
clusterEvalQ(cl, library(rms, quietly = TRUE))
clusterEvalQ(cl, library(mlt, quietly = TRUE))
clusterEvalQ(cl, library(tram, quietly = TRUE))
clusterEvalQ(cl, library(multcomp, quietly = TRUE))
clusterExport(cl,varlist=ls())

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) &
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt order=10
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order=10,
                         support = quantile(data$y, prob = c(.1, .9)),
                         add = c(-10, 10),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) &
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ###mlt survival
  data$y <- x+e
  data$yo <- data$y
  data$event <- x+e > 0
  data$y[!data$event] <- 0
  data$y <- Surv(data$y, data$event, type = "left")
  mlt_y_surv <- as.mlt(BoxCox(y ~ x, data, order=10, 
                              support = quantile(data$yo, prob = c(.1, .9)), 
                              add = c(-10, 10),
                              extrapolate = TRUE))
  
  ####beta
  res_mlt_surv$beta$est <- coef(mlt_y_surv)[['x']]
  res_mlt_surv$beta$bias <- res_mlt_surv$beta$est-beta
  res_mlt_surv$beta$se <- sqrt(vcov(mlt_y_surv)['x','x'])
  res_mlt_surv$beta$mse <- (res_mlt_surv$beta$est-beta)^2
  res_mlt_surv$beta$cover <- (res_mlt_surv$beta$est-qnorm(0.975)*res_mlt_surv$beta$se<=beta) &
    (res_mlt_surv$beta$est+qnorm(0.975)*res_mlt_surv$beta$se>=beta)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 200)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  p1 <- predict(object=mlt_y_surv, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y_surv, newdata=tmp2, type = "distribution")
  
  res_mlt_surv$loglikelihood <- sum(log(p2 - p1))
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  
  return(c(orm=res_orm,mlt=res_mlt,mlt_surv=res_mlt_surv))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=3)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=3)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt','mlt_surv')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)
all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=3)
rownames(ret_log) <- c('orm','mlt','mlt_surv')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n50"]] <- ret_beta
sum_log[["n50"]] <- ret_log

####n=100####
n <- 100
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) &
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt order=10
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order=10,
                         support = quantile(data$y, prob = c(.1, .9)),
                         add = c(-10, 10),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) &
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  
  ###mlt survival
  data$y <- x+e
  data$yo <- data$y
  data$event <- x+e > 0
  data$y[!data$event] <- 0
  data$y <- Surv(data$y, data$event, type = "left")
  mlt_y_surv <- as.mlt(BoxCox(y ~ x, data, order=10, 
                              support = quantile(data$yo, prob = c(.1, .9)), 
                              add = c(-10, 10),
                              extrapolate = TRUE))
  
  ####beta
  res_mlt_surv$beta$est <- coef(mlt_y_surv)[['x']]
  res_mlt_surv$beta$bias <- res_mlt_surv$beta$est-beta
  res_mlt_surv$beta$se <- sqrt(vcov(mlt_y_surv)['x','x'])
  res_mlt_surv$beta$mse <- (res_mlt_surv$beta$est-beta)^2
  res_mlt_surv$beta$cover <- (res_mlt_surv$beta$est-qnorm(0.975)*res_mlt_surv$beta$se<=beta) &
    (res_mlt_surv$beta$est+qnorm(0.975)*res_mlt_surv$beta$se>=beta)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 200)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  p1 <- predict(object=mlt_y_surv, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y_surv, newdata=tmp2, type = "distribution")
  
  res_mlt_surv$loglikelihood <- sum(log(p2 - p1))
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  
  return(c(orm=res_orm,mlt=res_mlt,mlt_surv=res_mlt_surv))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=3)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=3)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt','mlt_surv')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=3)
rownames(ret_log) <- c('orm','mlt','mlt_surv')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n100"]] <- ret_beta
sum_log[["n100"]] <- ret_log

####n=500####
n <- 500
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) &
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt order=10
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order=10,
                         support = quantile(data$y, prob = c(.1, .9)),
                         add = c(-10, 10),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) &
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  
  ###mlt survival
  data$y <- x+e
  data$yo <- data$y
  data$event <- x+e > 0
  data$y[!data$event] <- 0
  data$y <- Surv(data$y, data$event, type = "left")
  mlt_y_surv <- as.mlt(BoxCox(y ~ x, data, order=10, 
                              support = quantile(data$yo, prob = c(.1, .9)), 
                              add = c(-10, 10),
                              extrapolate = TRUE))
  
  ####beta
  res_mlt_surv$beta$est <- coef(mlt_y_surv)[['x']]
  res_mlt_surv$beta$bias <- res_mlt_surv$beta$est-beta
  res_mlt_surv$beta$se <- sqrt(vcov(mlt_y_surv)['x','x'])
  res_mlt_surv$beta$mse <- (res_mlt_surv$beta$est-beta)^2
  res_mlt_surv$beta$cover <- (res_mlt_surv$beta$est-qnorm(0.975)*res_mlt_surv$beta$se<=beta) &
    (res_mlt_surv$beta$est+qnorm(0.975)*res_mlt_surv$beta$se>=beta)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 200)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  p1 <- predict(object=mlt_y_surv, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y_surv, newdata=tmp2, type = "distribution")
  
  res_mlt_surv$loglikelihood <- sum(log(p2 - p1))
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  
  return(c(orm=res_orm,mlt=res_mlt,mlt_surv=res_mlt_surv))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=3)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=3)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt','mlt_surv')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)


all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=3)
rownames(ret_log) <- c('orm','mlt','mlt_surv')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n500"]] <- ret_beta
sum_log[["n500"]] <- ret_log


####n=1000####
n <- 1000
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) &
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  ###mlt order=10
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order=10,
                         support = quantile(data$y, prob = c(.1, .9)),
                         add = c(-10, 10),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) &
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ###mlt survival
  data$y <- x+e
  data$yo <- data$y
  data$event <- x+e > 0
  data$y[!data$event] <- 0
  data$y <- Surv(data$y, data$event, type = "left")
  mlt_y_surv <- as.mlt(BoxCox(y ~ x, data, order=10, 
                              support = quantile(data$yo, prob = c(.1, .9)), 
                              add = c(-10, 10),
                              extrapolate = TRUE))
  
  ####beta
  res_mlt_surv$beta$est <- coef(mlt_y_surv)[['x']]
  res_mlt_surv$beta$bias <- res_mlt_surv$beta$est-beta
  res_mlt_surv$beta$se <- sqrt(vcov(mlt_y_surv)['x','x'])
  res_mlt_surv$beta$mse <- (res_mlt_surv$beta$est-beta)^2
  res_mlt_surv$beta$cover <- (res_mlt_surv$beta$est-qnorm(0.975)*res_mlt_surv$beta$se<=beta) &
    (res_mlt_surv$beta$est+qnorm(0.975)*res_mlt_surv$beta$se>=beta)
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- ifelse((x+e)>0, x+e, 0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 200)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  p1 <- predict(object=mlt_y_surv, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y_surv, newdata=tmp2, type = "distribution")
  
  res_mlt_surv$loglikelihood <- sum(log(p2 - p1))
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  
  return(c(orm=res_orm,mlt=res_mlt,mlt_surv=res_mlt_surv))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=3)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=3)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt','mlt_surv')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=3)
rownames(ret_log) <- c('orm','mlt','mlt_surv')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n1000"]] <- ret_beta
sum_log[["n1000"]] <- ret_log


summary <- list('beta'=sum_beta,'log-likelihood'=sum_log)