#### Discretization - 50 categories (continuous) #####

####packages####
library(rms)
library(mlt)
library(tram)
library(multcomp)
library(parallel)
library(tidyverse)
library(magrittr)

####functions####
source('~/simulation/sim10000/sim_function.R')

###number of replication#####
reps <- 10000

####stoer the summary data
names <- c('n50','n100','n500','n1000')
sum_beta <- vector("list",4); names(sum_beta) <- names 
sum_mean <- vector("list",4); names(sum_mean) <- names
sum_log <- vector("list",4); names(sum_log) <- names #log-likelihood

###store result
res_orm <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),3),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),1),
             0)
names(res_orm) <- c('beta','mean_0','mean_1','loglikelihood')

res_mlt <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),3),
             rep(list(data.frame(est=0,bias=0,mse=0,cover=0)),1),
             0)
names(res_mlt) <- c('beta','mean_0','mean_1','loglikelihood')


###True values using simulation (m=mean,q=quantile,c=cdf)
set.seed(35)
beta <- 1
y_0 <- qchisq(pnorm(rnorm(1000000,0,1)+0),df=5)
y_1 <- qchisq(pnorm(rnorm(1000000,0,1)+1),df=5)
y_0 <- cut(y_0,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
levels(y_0) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
y_1 <- cut(y_1,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
levels(y_1) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)

true_1_m <- mean(as.numeric(y_1))
true_0_m <- mean(as.numeric(y_0))

####n=50####
n <- 50

####set up cluster###
cl <- makeCluster(detectCores()-1)
clusterEvalQ(cl, library(rms, quietly = TRUE))
clusterEvalQ(cl, library(mlt, quietly = TRUE))
clusterEvalQ(cl, library(tram, quietly = TRUE))
clusterEvalQ(cl, library(multcomp, quietly = TRUE))
clusterExport(cl,varlist=ls())

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  #category 0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
  # 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
  # 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  ####mean
  mean_orm <- mean.orm(model_orm,new.data = newdata, se=T)
  
  res_orm$mean_0$est <- mean_orm[,1][newdata[,'x']==0] 
  res_orm$mean_0$bias <- res_orm$mean_0$est-true_0_m
  res_orm$mean_0$se <- mean_orm[,2][newdata[,'x']==0]
  res_orm$mean_0$mse <- (res_orm$mean_0$est-true_0_m)^2
  res_orm$mean_0$cover <-  (res_orm$mean_0$est-qnorm(0.975)*res_orm$mean_0$se <=true_0_m) &
    (res_orm$mean_0$est+qnorm(0.975)*res_orm$mean_0$se >=true_0_m)
  
  res_orm$mean_1$est <- mean_orm[,1][newdata[,'x']==1] 
  res_orm$mean_1$bias <- res_orm$mean_1$est-true_1_m
  res_orm$mean_1$se <- mean_orm[,2][newdata[,'x']==1]
  res_orm$mean_1$mse <- (res_orm$mean_1$est-true_1_m)^2
  res_orm$mean_1$cover <-  (res_orm$mean_1$est-qnorm(0.975)*res_orm$mean_1$se <=true_1_m) &
    (res_orm$mean_1$est+qnorm(0.975)*res_orm$mean_1$se >=true_1_m)
  
  
  ###mlt order=5
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                         support = quantile(data$y, prob = c(.1, .9)), 
                         add = c(-15, 15),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ####mean
  mlt_y_mean <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                              support = c(min(data$y),max(data$y)), 
                              bounds=c(-Inf,Inf))) #to get basis at every data point mkgrid generates
  mean_mlt <- mean.mlt(mlt_y,mlt_y_mean,data,newdata)
  
  res_mlt$mean_0$est <- mean_mlt[,1][newdata[,'x']==0] 
  res_mlt$mean_0$bias <- res_mlt$mean_0$est-true_0_m
  res_mlt$mean_0$se <- mean_mlt[,2][newdata[,'x']==0]
  res_mlt$mean_0$mse <- (res_mlt$mean_0$est-true_0_m)^2
  res_mlt$mean_0$cover <-  (res_mlt$mean_0$est-qnorm(0.975)*res_mlt$mean_0$se <=true_0_m) &
    (res_mlt$mean_0$est+qnorm(0.975)*res_mlt$mean_0$se >=true_0_m)
  
  res_mlt$mean_1$est <- mean_mlt[,1][newdata[,'x']==1] 
  res_mlt$mean_1$bias <- res_mlt$mean_1$est-true_1_m
  res_mlt$mean_1$se <- mean_mlt[,2][newdata[,'x']==1]
  res_mlt$mean_1$mse <- (res_mlt$mean_1$est-true_1_m)^2
  res_mlt$mean_1$cover <-  (res_mlt$mean_1$est-qnorm(0.975)*res_mlt$mean_1$se <=true_1_m) &
    (res_mlt$mean_1$est+qnorm(0.975)*res_mlt$mean_1$se >=true_1_m)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 15)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  return(c(orm=res_orm,mlt=res_mlt))
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_mean <- result %>% dplyr::select(contains('mean')) 
ret_mean <- matrix(apply(all_mean,2,mean),byrow=T,nrow=2*2)
ret_mean2 <- matrix(apply(all_mean,2,sd),byrow=T,nrow=2*2)
ret_mean[,1] <- ret_mean2[,1]
colnames(ret_mean) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_mean) <- c('orm_0','orm_1','mlt_0','mlt_1')
ret_mean <- ret_mean  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE) %>% round(5)
ret_mean <- ret_mean[c(1,3,2,4),]

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=2)
rownames(ret_log) <- c('orm','mlt')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n50"]] <- ret_beta
sum_mean[["n50"]] <- ret_mean
sum_log[["n50"]] <- ret_log

####n=100####
n <- 100
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  #category 0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
  # 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
  # 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  ####mean
  mean_orm <- mean.orm(model_orm,new.data = newdata, se=T)
  
  res_orm$mean_0$est <- mean_orm[,1][newdata[,'x']==0] 
  res_orm$mean_0$bias <- res_orm$mean_0$est-true_0_m
  res_orm$mean_0$se <- mean_orm[,2][newdata[,'x']==0]
  res_orm$mean_0$mse <- (res_orm$mean_0$est-true_0_m)^2
  res_orm$mean_0$cover <-  (res_orm$mean_0$est-qnorm(0.975)*res_orm$mean_0$se <=true_0_m) &
    (res_orm$mean_0$est+qnorm(0.975)*res_orm$mean_0$se >=true_0_m)
  
  res_orm$mean_1$est <- mean_orm[,1][newdata[,'x']==1] 
  res_orm$mean_1$bias <- res_orm$mean_1$est-true_1_m
  res_orm$mean_1$se <- mean_orm[,2][newdata[,'x']==1]
  res_orm$mean_1$mse <- (res_orm$mean_1$est-true_1_m)^2
  res_orm$mean_1$cover <-  (res_orm$mean_1$est-qnorm(0.975)*res_orm$mean_1$se <=true_1_m) &
    (res_orm$mean_1$est+qnorm(0.975)*res_orm$mean_1$se >=true_1_m)
  
  
  ###mlt order=5
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                         support = quantile(data$y, prob = c(.1, .9)), 
                         add = c(-15, 15),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ####mean
  mlt_y_mean <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                              support = c(min(data$y),max(data$y)), 
                              bounds=c(-Inf,Inf))) #to get basis at every data point mkgrid generates
  mean_mlt <- mean.mlt(mlt_y,mlt_y_mean,data,newdata)
  
  res_mlt$mean_0$est <- mean_mlt[,1][newdata[,'x']==0] 
  res_mlt$mean_0$bias <- res_mlt$mean_0$est-true_0_m
  res_mlt$mean_0$se <- mean_mlt[,2][newdata[,'x']==0]
  res_mlt$mean_0$mse <- (res_mlt$mean_0$est-true_0_m)^2
  res_mlt$mean_0$cover <-  (res_mlt$mean_0$est-qnorm(0.975)*res_mlt$mean_0$se <=true_0_m) &
    (res_mlt$mean_0$est+qnorm(0.975)*res_mlt$mean_0$se >=true_0_m)
  
  res_mlt$mean_1$est <- mean_mlt[,1][newdata[,'x']==1] 
  res_mlt$mean_1$bias <- res_mlt$mean_1$est-true_1_m
  res_mlt$mean_1$se <- mean_mlt[,2][newdata[,'x']==1]
  res_mlt$mean_1$mse <- (res_mlt$mean_1$est-true_1_m)^2
  res_mlt$mean_1$cover <-  (res_mlt$mean_1$est-qnorm(0.975)*res_mlt$mean_1$se <=true_1_m) &
    (res_mlt$mean_1$est+qnorm(0.975)*res_mlt$mean_1$se >=true_1_m)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 15)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  return(c(orm=res_orm,mlt=res_mlt))
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_mean <- result %>% dplyr::select(contains('mean')) 
ret_mean <- matrix(apply(all_mean,2,mean),byrow=T,nrow=2*2)
ret_mean2 <- matrix(apply(all_mean,2,sd),byrow=T,nrow=2*2)
ret_mean[,1] <- ret_mean2[,1]
colnames(ret_mean) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_mean) <- c('orm_0','orm_1','mlt_0','mlt_1')
ret_mean <- ret_mean  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE) %>% round(5)
ret_mean <- ret_mean[c(1,3,2,4),]

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=2)
rownames(ret_log) <- c('orm','mlt')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n100"]] <- ret_beta
sum_mean[["n100"]] <- ret_mean
sum_log[["n100"]] <- ret_log

####n=500####
n <- 500
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  #category 0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
  # 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
  # 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  ####mean
  mean_orm <- mean.orm(model_orm,new.data = newdata, se=T)
  
  res_orm$mean_0$est <- mean_orm[,1][newdata[,'x']==0] 
  res_orm$mean_0$bias <- res_orm$mean_0$est-true_0_m
  res_orm$mean_0$se <- mean_orm[,2][newdata[,'x']==0]
  res_orm$mean_0$mse <- (res_orm$mean_0$est-true_0_m)^2
  res_orm$mean_0$cover <-  (res_orm$mean_0$est-qnorm(0.975)*res_orm$mean_0$se <=true_0_m) &
    (res_orm$mean_0$est+qnorm(0.975)*res_orm$mean_0$se >=true_0_m)
  
  res_orm$mean_1$est <- mean_orm[,1][newdata[,'x']==1] 
  res_orm$mean_1$bias <- res_orm$mean_1$est-true_1_m
  res_orm$mean_1$se <- mean_orm[,2][newdata[,'x']==1]
  res_orm$mean_1$mse <- (res_orm$mean_1$est-true_1_m)^2
  res_orm$mean_1$cover <-  (res_orm$mean_1$est-qnorm(0.975)*res_orm$mean_1$se <=true_1_m) &
    (res_orm$mean_1$est+qnorm(0.975)*res_orm$mean_1$se >=true_1_m)
  
  
  ###mlt order=5
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                         support = quantile(data$y, prob = c(.1, .9)), 
                         add = c(-15, 15),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ####mean
  mlt_y_mean <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                              support = c(min(data$y),max(data$y)), 
                              bounds=c(-Inf,Inf))) #to get basis at every data point mkgrid generates
  mean_mlt <- mean.mlt(mlt_y,mlt_y_mean,data,newdata)
  
  res_mlt$mean_0$est <- mean_mlt[,1][newdata[,'x']==0] 
  res_mlt$mean_0$bias <- res_mlt$mean_0$est-true_0_m
  res_mlt$mean_0$se <- mean_mlt[,2][newdata[,'x']==0]
  res_mlt$mean_0$mse <- (res_mlt$mean_0$est-true_0_m)^2
  res_mlt$mean_0$cover <-  (res_mlt$mean_0$est-qnorm(0.975)*res_mlt$mean_0$se <=true_0_m) &
    (res_mlt$mean_0$est+qnorm(0.975)*res_mlt$mean_0$se >=true_0_m)
  
  res_mlt$mean_1$est <- mean_mlt[,1][newdata[,'x']==1] 
  res_mlt$mean_1$bias <- res_mlt$mean_1$est-true_1_m
  res_mlt$mean_1$se <- mean_mlt[,2][newdata[,'x']==1]
  res_mlt$mean_1$mse <- (res_mlt$mean_1$est-true_1_m)^2
  res_mlt$mean_1$cover <-  (res_mlt$mean_1$est-qnorm(0.975)*res_mlt$mean_1$se <=true_1_m) &
    (res_mlt$mean_1$est+qnorm(0.975)*res_mlt$mean_1$se >=true_1_m)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 15)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  return(c(orm=res_orm,mlt=res_mlt))
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_mean <- result %>% dplyr::select(contains('mean')) 
ret_mean <- matrix(apply(all_mean,2,mean),byrow=T,nrow=2*2)
ret_mean2 <- matrix(apply(all_mean,2,sd),byrow=T,nrow=2*2)
ret_mean[,1] <- ret_mean2[,1]
colnames(ret_mean) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_mean) <- c('orm_0','orm_1','mlt_0','mlt_1')
ret_mean <- ret_mean  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE) %>% round(5)
ret_mean <- ret_mean[c(1,3,2,4),]

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=2)
rownames(ret_log) <- c('orm','mlt')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n500"]] <- ret_beta
sum_mean[["n500"]] <- ret_mean
sum_log[["n500"]] <- ret_log


####n=1000####
n <- 1000
clusterExport(cl, c("n"))

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  #category 0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
  # 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
  # 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  ####mean
  mean_orm <- mean.orm(model_orm,new.data = newdata, se=T)
  
  res_orm$mean_0$est <- mean_orm[,1][newdata[,'x']==0] 
  res_orm$mean_0$bias <- res_orm$mean_0$est-true_0_m
  res_orm$mean_0$se <- mean_orm[,2][newdata[,'x']==0]
  res_orm$mean_0$mse <- (res_orm$mean_0$est-true_0_m)^2
  res_orm$mean_0$cover <-  (res_orm$mean_0$est-qnorm(0.975)*res_orm$mean_0$se <=true_0_m) &
    (res_orm$mean_0$est+qnorm(0.975)*res_orm$mean_0$se >=true_0_m)
  
  res_orm$mean_1$est <- mean_orm[,1][newdata[,'x']==1] 
  res_orm$mean_1$bias <- res_orm$mean_1$est-true_1_m
  res_orm$mean_1$se <- mean_orm[,2][newdata[,'x']==1]
  res_orm$mean_1$mse <- (res_orm$mean_1$est-true_1_m)^2
  res_orm$mean_1$cover <-  (res_orm$mean_1$est-qnorm(0.975)*res_orm$mean_1$se <=true_1_m) &
    (res_orm$mean_1$est+qnorm(0.975)*res_orm$mean_1$se >=true_1_m)
  
 
  ###mlt order=5
  mlt_y <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                         support = quantile(data$y, prob = c(.1, .9)), 
                         add = c(-15, 15),
                         extrapolate = TRUE))
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  ####mean
  mlt_y_mean <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                              support = c(min(data$y),max(data$y)), 
                              bounds=c(-Inf,Inf))) #to get basis at every data point mkgrid generates
  mean_mlt <- mean.mlt(mlt_y,mlt_y_mean,data,newdata)
  
  res_mlt$mean_0$est <- mean_mlt[,1][newdata[,'x']==0] 
  res_mlt$mean_0$bias <- res_mlt$mean_0$est-true_0_m
  res_mlt$mean_0$se <- mean_mlt[,2][newdata[,'x']==0]
  res_mlt$mean_0$mse <- (res_mlt$mean_0$est-true_0_m)^2
  res_mlt$mean_0$cover <-  (res_mlt$mean_0$est-qnorm(0.975)*res_mlt$mean_0$se <=true_0_m) &
    (res_mlt$mean_0$est+qnorm(0.975)*res_mlt$mean_0$se >=true_0_m)
  
  res_mlt$mean_1$est <- mean_mlt[,1][newdata[,'x']==1] 
  res_mlt$mean_1$bias <- res_mlt$mean_1$est-true_1_m
  res_mlt$mean_1$se <- mean_mlt[,2][newdata[,'x']==1]
  res_mlt$mean_1$mse <- (res_mlt$mean_1$est-true_1_m)^2
  res_mlt$mean_1$cover <-  (res_mlt$mean_1$est-qnorm(0.975)*res_mlt$mean_1$se <=true_1_m) &
    (res_mlt$mean_1$est+qnorm(0.975)*res_mlt$mean_1$se >=true_1_m)
  
  
  ###out-of-sample log-likelihood
  ####generate test sample
  set.seed(2)
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- cut(y,c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
               4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
               9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0,Inf))
  levels(y) <- c(0, 1, 1.4, 1.6, 1.9, 2.1, 2.3, 2.5, 2.7, 2.9, 3.1, 3.3, 3.5, 3.7, 3.8, 4.0, 4.2, 4.4, 4.6,
                 4.8, 5.0, 5.1, 5.3, 5.5,5.7, 5.9, 6.1, 6.4,  6.6, 6.8, 7.0, 7.3, 7.6, 7.8, 8.2, 8.5, 8.8, 9.0,
                 9.4, 9.8, 10.2, 10.6, 11.0, 11.5, 12.0, 13.0, 13.5, 14.5, 16.0, 18.0)
  test <- data.frame('x'=x,'y'=y)
  test <- as.data.frame(sapply(test,as.numeric))
  
  ###mlt
  brk <- model_orm$yunique
  ### -/+10 for values not in range(o$yunique)
  brk <- c(brk[1] - 15, brk, rev(brk)[1] + 15)
  test$yc <- unclass(cut(test$y, breaks = brk))
  
  tmp1 <- tmp2 <- test
  tmp1$y <- brk[tmp1$yc]
  tmp2$y <- brk[tmp2$yc + 1]
  
  p1 <- predict(object=mlt_y, newdata=tmp1, type = "distribution")
  p2 <- predict(object=mlt_y, newdata=tmp2, type = "distribution")
  
  res_mlt$loglikelihood <- sum(log(p2 - p1))
  
  
  ###orm
  test$yc <- unclass(cut(test$y, breaks = brk, right=F)) - 1
  test$yc[test$yc == 0] = 1
  
  p <- predict(model_orm, newdata = test, type = "fitted")
  ### first cut-off is missing
  ### add one and zero probabilities to survivor fct
  p <- cbind(1, p, 0)
  p1 <- p[cbind(1:nrow(test), test$yc + 1)]
  p2 <- p[cbind(1:nrow(test), test$yc)]
  ### log-likelihood of sum of log-probabilities
  res_orm$loglikelihood <- sum(log(p2 - p1))
  
  return(c(orm=res_orm,mlt=res_mlt))
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% dplyr::select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% dplyr::select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

all_mean <- result %>% dplyr::select(contains('mean')) 
ret_mean <- matrix(apply(all_mean,2,mean),byrow=T,nrow=2*2)
ret_mean2 <- matrix(apply(all_mean,2,sd),byrow=T,nrow=2*2)
ret_mean[,1] <- ret_mean2[,1]
colnames(ret_mean) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_mean) <- c('orm_0','orm_1','mlt_0','mlt_1')
ret_mean <- ret_mean  %>%  as.data.frame() %>% dplyr::select(Bias,MSE,Cover,SD_Est,Mean_SE) %>% round(5)
ret_mean <- ret_mean[c(1,3,2,4),]

all_log <- result %>% dplyr::select(contains('log')) 
ret_log <- matrix(apply(all_log,2,mean),nrow=2)
rownames(ret_log) <- c('orm','mlt')
colnames(ret_log) <- 'log-likelihood'
ret_log <- ret_log %>% as.data.frame()

sum_beta[["n1000"]] <- ret_beta
sum_mean[["n1000"]] <- ret_mean
sum_log[["n1000"]] <- ret_log


summary <- list('beta'=sum_beta,'mean'=sum_mean,'log-likelihood'=sum_log)