### Discretization - 5 categories (ordered factor) ###

####packages####
library(rms)
library(mlt)
library(parallel)
library(tidyverse)

####functions####
source('~/simulation/sim10000/sim_function.R')

####stoer the summary data
names <- c('n50','n100','n500','n1000')
sum_beta <- vector("list",4); names(sum_beta) <- names 

###store result
res_orm <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),1))
names(res_orm) <- c('beta')

res_mlt <- c(rep(list(data.frame(est=0,bias=0,se=0,mse=0,cover=0)),1))
names(res_mlt) <- c('beta')

###True values using simulation (m=mean,q=quantile,c=cdf)
beta <- 1

####n=50####
n <- 50

####set up cluster###
cl <- makeCluster(detectCores()-2)
clusterEvalQ(cl, library(rms, quietly = TRUE))
clusterEvalQ(cl, library(mlt, quietly = TRUE))
#clusterExport(cl,varlist=ls(),envir = environment())
clusterExport(cl,varlist=ls())

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  #category 0 3 5 7 10
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- ifelse(y<3,0,ifelse(y<5,3,ifelse(y<7,5,ifelse(y<10,7,10))))
  data <- data.frame('x'=x,'y'=y)
  data_orm <- as.data.frame(sapply(data,as.numeric))
  y <- factor(y,ordered=T)
  data_mlt <- data.frame('x'=x,'y'=y)
  new <- as.data.frame('x'=c(0,1))
  colnames(new) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt 
  nl <- nlevels(data_mlt$y)
  b_y <- as.basis(~y,data=data_mlt,remove_intercept=T,contrasts.arg=list(y=function(n) contr.treatment(n,nl)),ui=diff(diag(nl-1)),ci=rep(0,nl-2))
  b_x <- as.basis(~x,data=data_mlt,remove_intercept=T)
  ctm_y <- ctm(response=b_y,shift=b_x,todistr = 'Normal')
  mlt_y <- mlt(model=ctm_y,data=data_mlt,scale=T)
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]*(-1)
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  return(c(orm=res_orm,mlt=res_mlt))
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

sum_beta[["n50"]] <- ret_beta

####n=100####
n <- 100
clusterExport(cl,'n')

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  #category 0 3 5 7 10
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- ifelse(y<3,0,ifelse(y<5,3,ifelse(y<7,5,ifelse(y<10,7,10))))
  data <- data.frame('x'=x,'y'=y)
  data_orm <- as.data.frame(sapply(data,as.numeric))
  y <- factor(y,ordered=T)
  data_mlt <- data.frame('x'=x,'y'=y)
  new <- as.data.frame('x'=c(0,1))
  colnames(new) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt
  nl <- nlevels(data_mlt$y)
  b_y <- as.basis(~y,data=data_mlt,remove_intercept=T,contrasts.arg=list(y=function(n) contr.treatment(n,nl)),ui=diff(diag(nl-1)),ci=rep(0,nl-2))
  b_x <- as.basis(~x,data=data_mlt,remove_intercept=T)
  ctm_y <- ctm(response=b_y,shift=b_x,todistr = 'Normal')
  mlt_y <- mlt(model=ctm_y,data=data_mlt,scale=T)
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]*(-1)
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  return(c(orm=res_orm,mlt=res_mlt))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

sum_beta[["n100"]] <- ret_beta

####n=500####
n <- 500
clusterExport(cl,'n')

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  #category 0 3 5 7 10
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- ifelse(y<3,0,ifelse(y<5,3,ifelse(y<7,5,ifelse(y<10,7,10))))
  data <- data.frame('x'=x,'y'=y)
  data_orm <- as.data.frame(sapply(data,as.numeric))
  y <- factor(y,ordered=T)
  data_mlt <- data.frame('x'=x,'y'=y)
  new <- as.data.frame('x'=c(0,1))
  colnames(new) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt
  nl <- nlevels(data_mlt$y)
  b_y <- as.basis(~y,data=data_mlt,remove_intercept=T,contrasts.arg=list(y=function(n) contr.treatment(n,nl)),ui=diff(diag(nl-1)),ci=rep(0,nl-2))
  b_x <- as.basis(~x,data=data_mlt,remove_intercept=T)
  ctm_y <- ctm(response=b_y,shift=b_x,todistr = 'Normal')
  mlt_y <- mlt(model=ctm_y,data=data_mlt,scale=T)
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]*(-1)
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  return(c(orm=res_orm,mlt=res_mlt))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

sum_beta[["n500"]] <- ret_beta

####n=1000####
n <- 1000
clusterExport(cl,'n')

result <- parLapply(cl, 1:reps,function(i)
{
  set.seed(i)
  #generate data
  #category 0 3 5 7 10
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  y <- ifelse(y<3,0,ifelse(y<5,3,ifelse(y<7,5,ifelse(y<10,7,10))))
  data <- data.frame('x'=x,'y'=y)
  data_orm <- as.data.frame(sapply(data,as.numeric))
  y <- factor(y,ordered=T)
  data_mlt <- data.frame('x'=x,'y'=y)
  new <- as.data.frame('x'=c(0,1))
  colnames(new) <- 'x'
  
  ###orm
  model_orm <- orm(y~x, data=data,family=probit) 
  
  ####beta
  res_orm$beta$est <- coef(model_orm)[['x']]
  res_orm$beta$bias <- res_orm$beta$est-beta
  res_orm$beta$se <- sqrt(model_orm$var['x','x'])
  res_orm$beta$mse <- (res_orm$beta$est-beta)^2
  res_orm$beta$cover <- (res_orm$beta$est-qnorm(0.975)*res_orm$beta$se<=beta) & 
    (res_orm$beta$est+qnorm(0.975)*res_orm$beta$se>=beta)
  
  
  ###mlt 
  nl <- nlevels(data_mlt$y)
  b_y <- as.basis(~y,data=data_mlt,remove_intercept=T,contrasts.arg=list(y=function(n) contr.treatment(n,nl)),ui=diff(diag(nl-1)),ci=rep(0,nl-2))
  b_x <- as.basis(~x,data=data_mlt,remove_intercept=T)
  ctm_y <- ctm(response=b_y,shift=b_x,todistr = 'Normal')
  mlt_y <- mlt(model=ctm_y,data=data_mlt,scale=T)
  
  ####beta
  res_mlt$beta$est <- coef(mlt_y)[['x']]*(-1)
  res_mlt$beta$bias <- res_mlt$beta$est-beta
  res_mlt$beta$se <- sqrt(vcov(mlt_y)['x','x'])
  res_mlt$beta$mse <- (res_mlt$beta$est-beta)^2
  res_mlt$beta$cover <- (res_mlt$beta$est-qnorm(0.975)*res_mlt$beta$se<=beta) & 
    (res_mlt$beta$est+qnorm(0.975)*res_mlt$beta$se>=beta)
  
  return(c(orm=res_orm,mlt=res_mlt))
  
  
})

result <- as.data.frame(do.call('rbind',lapply(result,unlist)))

all_beta <- result %>% select(contains('beta')) 
ret_beta <- matrix(apply(all_beta,2,mean),byrow=T,nrow=2)
ret_beta2 <- matrix(apply(all_beta,2,sd),byrow=T,nrow=2)
ret_beta[,1] <- ret_beta2[,1]
ret_beta <- as.data.frame(ret_beta)
colnames(ret_beta) <- c('SD_Est','Bias','Mean_SE','MSE','Cover')
rownames(ret_beta) <- c('orm','mlt')
ret_beta$Bias_CI_lb <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.025))
ret_beta$Bias_CI_ub <- (all_beta %>% select(contains('est'))-beta) %>% apply(.,2,quantile,probs=c(0.975))
ret_beta <- ret_beta  %>%  as.data.frame() %>% select(Bias,MSE,Cover,SD_Est,Mean_SE,Bias_CI_lb,Bias_CI_ub) %>% round(5)

sum_beta[["n1000"]] <- ret_beta

summary <- list('beta'=sum_beta)
