#### time #####

####packages####
library(rms)
library(mlt)
library(tram)
library(multcomp)
library(parallel)
library(tidyverse)
library(magrittr)

####functions####
source('~/simulation/sim10000/sim_function.R')

###number of replication#####
reps <- 100

###values for quantile and cdf
val_q <- c(0.1, 0.5, 0.8)
val_c <- 5 #cdf

#store result
orm_time <- list(data.frame(elapsed=0))
mlt5_time <- list(data.frame(elapsed=0))
mlt10_time <- list(data.frame(elapsed=0))

####n=10000####
n <- 10000

####set up cluster###
cl <- makeCluster(1)
clusterEvalQ(cl, library(rms, quietly = TRUE))
clusterEvalQ(cl, library(mlt, quietly = TRUE))
clusterEvalQ(cl, library(tram, quietly = TRUE))
clusterEvalQ(cl, library(multcomp, quietly = TRUE))
clusterExport(cl,varlist=ls())

result <- parLapply(cl, 1:reps,function(i)
{ 
  set.seed(i)
  #generate data
  x <- rbinom(n,1,p=0.5)
  e <- rnorm(n,0,1)
  y <- qchisq(pnorm(x+e),df=5)
  data <- data.frame('x'=x,'y'=y)
  data <- as.data.frame(sapply(data,as.numeric))
  newdata <- as.data.frame('x'=c(0,1))
  colnames(newdata) <- 'x'
  
  ###orm
  start <- proc.time()
  model_orm <- orm(y~x, data=data,family=probit) 
  
  orm_time <- (proc.time() - start)[3]
  
  ###mlt order=5
  start <- proc.time()
  mlt_y_5 <- as.mlt(BoxCox(y ~ x, data, order = 5, 
                           support = quantile(data$y, prob = c(.1, .9)), 
                           add = c(-15, 15),
                           extrapolate = TRUE))
  
  mlt5_time <- (proc.time() - start)[3]
  
  ###mlt order=10
  start <- proc.time()
  mlt_y_10 <- as.mlt(BoxCox(y ~ x, data, order = 10, 
                            support = quantile(data$y, prob = c(.1, .9)), 
                            add = c(-15, 15),
                            extrapolate = TRUE))
  
  mlt10_time <- (proc.time() - start)[3]
  
  return(c(orm=orm_time,mlt_5=mlt5_time,mlt_10=mlt10_time))
  
})

result_10000 <- as.data.frame(do.call('rbind',lapply(result,unlist)))


time <- list('n10000'=result_10000)
